function [searchImage, searchPos, targetState, targetAppModel] = initTargetAppModel(imgdat, region)

global opts

if(numel(region)==8)
    % polygon format
    [cx, cy, w, h] = getAxisAlignedBB(region);
    opts.rotateMode = true;
else
    x = region(1);
    y = region(2);
    w = region(3);
    h = region(4);
    cx = x+w/2;
    cy = y+h/2;
    opts.rotateMode = false;
end

%% init_pos is the centre of the initial bounding box
targetAppModel.hog_cell_size = opts.hog_cell_size;
targetAppModel.fixed_area = opts.fixed_area;           % standard area to which we resize the target
targetAppModel.inner_padding = opts.inner_padding;             % defines inner area used to sample colors from the foreground
targetAppModel.output_sigma_factor = opts.output_sigma_factor;             % standard deviation for the desired translation filter output
targetAppModel.lambda = opts.lambda;                                   % regularization weight
targetAppModel.learning_rate_cf = opts.learning_rate_cf;            % HOG model learning rate
targetAppModel.hog_scale_cell_size = opts.hog_scale_cell_size;            
targetAppModel.learning_rate_scale = opts.learning_rate_scale;
targetAppModel.scale_sigma_factor = opts.scale_sigma_factor;
targetAppModel.num_scales = opts.num_scales;
targetAppModel.scale_model_factor = opts.scale_model_factor;
targetAppModel.scale_step = opts.scale_step;
targetAppModel.scale_model_max_area = opts.scale_model_max_area;

targetAppModel.targetCenter = [cy cx];
targetAppModel.targetSize = round([h w]);
[targetAppModel, targetAppModel.bg_area, targetAppModel.fg_area, targetAppModel.area_resize_factor] = initializeAllAreas(imgdat, targetAppModel);

%% initialize the corelation filter
targetCenter = targetAppModel.targetCenter;
targetSize = targetAppModel.targetSize;
% Hann (cosine) window
targetAppModel.hann_window = single(hann(targetAppModel.cf_response_size(1)) * hann(targetAppModel.cf_response_size(2))');
% gaussian-shaped desired response, centred in (1,1)
% bandwidth proportional to target size
output_sigma = sqrt(prod(targetAppModel.norm_target_sz)) * targetAppModel.output_sigma_factor / targetAppModel.hog_cell_size;
y = gaussianResponse(targetAppModel.cf_response_size, output_sigma);
targetAppModel.yf = fft2(y);
% initialize scale adaption
targetAppModel.scale_factor = 1;
targetAppModel.base_target_sz = targetSize;
scale_sigma = sqrt(targetAppModel.num_scales) * targetAppModel.scale_sigma_factor;
ss = (1:targetAppModel.num_scales) - ceil(targetAppModel.num_scales/2);
ys = exp(-0.5 * (ss.^2) / scale_sigma^2);
targetAppModel.ysf = single(fft(ys));
if mod(targetAppModel.num_scales,2) == 0
    scale_window = single(hann(targetAppModel.num_scales+1));
    targetAppModel.scale_window = scale_window(2:end);
else
    targetAppModel.scale_window = single(hann(targetAppModel.num_scales));
end;

ss = 1:targetAppModel.num_scales;
targetAppModel.scale_factors = targetAppModel.scale_step.^(ceil(targetAppModel.num_scales/2) - ss);

if(targetAppModel.scale_model_factor^2 * prod(targetAppModel.norm_target_sz) > targetAppModel.scale_model_max_area)
    targetAppModel.scale_model_factor = sqrt(targetAppModel.scale_model_max_area/prod(targetAppModel.norm_target_sz));
end

targetAppModel.scale_model_sz = floor(targetAppModel.norm_target_sz * targetAppModel.scale_model_factor);
% find maximum and minimum scales
targetAppModel.min_scale_factor = targetAppModel.scale_step ^ceil(log(max(5 ./ targetAppModel.bg_area)) / log(targetAppModel.scale_step));
targetAppModel.max_scale_factor = targetAppModel.scale_step ^floor(log(min([size(imgdat,1) size(imgdat,2)] ./ targetSize)) / log(targetAppModel.scale_step));

% patch of the target + padding
[searchImage, searchPos] = getSubwindow(imgdat, targetCenter, targetAppModel.norm_bg_area, targetAppModel.bg_area);
% compute feature map, of cf_response_size
xt = getFeatureMap(searchImage, targetAppModel.cf_response_size, targetAppModel.hog_cell_size);
% apply Hann window
xt = bsxfun(@times, targetAppModel.hann_window, xt);
% compute FFT
xtf = fft2(xt);
% Compute expectations over circular shifts, therefore divide by number of pixels.
targetAppModel.hf_num = bsxfun(@times, conj(targetAppModel.yf), xtf) / prod(targetAppModel.cf_response_size);
targetAppModel.hf_den = (conj(xtf) .* xtf) / prod(targetAppModel.cf_response_size);

im_patch_scale = getScaleSubwindow(imgdat, targetCenter, targetSize, targetAppModel.scale_factor*targetAppModel.scale_factors, targetAppModel.scale_window, targetAppModel.scale_model_sz, targetAppModel.hog_scale_cell_size);
xsf = fft(im_patch_scale,[],2);
targetAppModel.sf_num = bsxfun(@times, targetAppModel.ysf, conj(xsf));
targetAppModel.sf_den = sum(xsf .* conj(xsf), 1);

% obtain the target state
if(opts.rotateMode)    
    targetPos = zeros(1, 8);
    targetPos(1) = targetCenter(2) - targetSize(2)/2;     
    targetPos(2) = targetCenter(1) + targetSize(1)/2; 
    targetPos(3) = targetCenter(2) - targetSize(2)/2;
    targetPos(4) = targetCenter(1) - targetSize(1)/2;
    targetPos(5) = targetCenter(2) + targetSize(2)/2; 
    targetPos(6) = targetCenter(1) - targetSize(1)/2;
    targetPos(7) = targetCenter(2) + targetSize(2)/2;
    targetPos(8) = targetCenter(1) + targetSize(1)/2;
else
    targetPos = [targetCenter([2,1]) - targetSize([2,1])/2, targetSize([2,1])];
end
targetState.targetPos = targetPos;
targetState.targetCenter = targetCenter;
targetState.targetSize = targetSize;